/*
Pong Mania- A 3D OpenGL pong game.
Copyright (C) 2001  Steve Wortham

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/



/*
 		Pong Mania
 		by: Steve Wortham
 		website: www.stevescomp.atfreeweb.com
 		email: steve.wortham@juno.com
 		
*/

#include <math.h>		// Header File For Windows
#include <windows.h>										// Header File For Windows
#include <stdio.h>											// Header File For Standard Input / Output
#include <stdarg.h>											// Header File For Variable Argument Routines
#include <gl\gl.h>											// Header File For The OpenGL32 Library
#include <gl\glu.h>											// Header File For The GLu32 Library
#include <gl\glaux.h>										// Header File For The Glaux Library

bool up;
bool dn;
bool rn;
bool esc;
//#include <windows.h>		// Header File For Windows
//#include <stdio.h>			// Header File For Standard Input/Output
//#include <gl\gl.h>			// Header File For The OpenGL32 Library
//#include <gl\glu.h>			// Header File For The GLu32 Library
//#include <gl\glaux.h>		// Header File For The Glaux Library

HDC		hDC=NULL;			// Private GDI Device Context
HGLRC	hRC=NULL;			// Permanent Rendering Context
HWND	hWnd=NULL;			// Holds Our Window Handle

bool DONE = false;
bool MENU = true;
int item = 1;
bool Selection = false;
bool PlayAgain = true;
bool Options = false;
bool Exit = false;


bool	keys[256];			// Array Used For The Keyboard Routine
bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default

GLUquadricObj *quadratic;	// Storage For Our Quadratic Objects ( NEW )

// Absolute value function(comes in handy later).
inline GLfloat ABS(GLfloat A)
{
  if (A < 0)
  A = -A; 
  return A;
}
// Hypotenuse Function
inline GLfloat Hypot(GLfloat a, GLfloat b)
{
  return sqrt((a*a)+(b*b));
}

GLfloat LightAmbient[]=		{ 0.2f, 0.2f, 0.2f, 1.0f };
GLfloat LightDiffuse[]=		{ 0.5f, 0.5f, 0.5f, 1.0f };
GLfloat LightSpecular[]=	{ 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightPosition[]=	{ 0.0f, 0.0f, -11.0f, 1.0f };

GLuint	fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR };	// Storage For Three Types Of Fog
GLuint	fogfilter = 0;								// Which Fog Mode To Use 
GLfloat	fogColor[4] = {.7f, .7f, .8f, 0};		            // Fog Color

GLuint	filter;				// Which Filter To Use
GLuint	texture[15];			// Storage For 3 Textures
GLuint  object=0;			// Which Object To Draw (NEW)

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc


								// Storage For 96 Characters
// Create A Structure For The Timer Information
struct
{
  __int64       frequency;          // Timer Frequency
  float            resolution;          // Timer Resolution
  unsigned long mm_timer_start;     
  
  // Multimedia Timer Start Value
  unsigned long mm_timer_elapsed;      // Multimedia Timer Elapsed Time
  bool   performance_timer;    
  
  // Using The Performance Timer?
  __int64       performance_timer_start;      // Performance Timer Start Value
  __int64       performance_timer_elapsed; // Performance Timer Elapsed Time
} timer;

// Initialize Our Timer
void TimerInit(void)
{
     memset(&timer, 0, sizeof(timer));   
 // Clear Our Timer Structure
     // Check To See If A Performance Counter Is Available
     // If One Is Available The Timer Frequency Will Be Updated
     if (!QueryPerformanceFrequency((LARGE_INTEGER *) &timer.frequency))
     {
          // No Performace Counter Available
          timer.performance_timer = FALSE;                      // Set Performance Timer To FALSE
          timer.mm_timer_start = timeGetTime();                 // Use timeGetTime()
          timer.resolution  = 1.0f/1000.0f;                           // Set Our Timer Resolution To .001f
          timer.frequency   = 1000;                                     // Set Our Timer Frequency To 1000
          timer.mm_timer_elapsed = timer.mm_timer_start; // Set The Elapsed Time
     }
     else
     {
          // Performance Counter Is Available, Use It Instead Of The Multimedia Timer
          // Get The Current Time And Store It In performance_timer_start
          QueryPerformanceCounter((LARGE_INTEGER *) &timer.performance_timer_start);
          timer.performance_timer   = TRUE;    // Set Performance Timer To TRUE
          // Calculate The Timer Resolution Using The Timer Frequency
          timer.resolution    = (float) (((double)1.0f)/((double)timer.frequency));
          // Set The Elapsed Time To The Current Time
          timer.performance_timer_elapsed = timer.performance_timer_start;
     }
}

// Get Time In Milliseconds
float TimerGetTime()
{
     __int64 time;                                  // 'time' Will Hold A 64 Bit Integer
     if (timer.performance_timer)           // Are We Using The Performance Timer?
     {
          QueryPerformanceCounter((LARGE_INTEGER *) &time); // Current Performance Time
          // Return The Time Elapsed since TimerInit was called
          return ( (float) ( time - timer.performance_timer_start) * timer.resolution)*1000.0f;
     }
     else
     {
          // Return The Time Elapsed since TimerInit was called
          return ( (float) ( timeGetTime() - timer.mm_timer_start) * timer.resolution)*1000.0f;
     }
}


AUX_RGBImageRec *LoadBMP(char *Filename)				// Loads A Bitmap Image
{
	FILE *File=NULL;									// File Handle

	if (!Filename)										// Make Sure A Filename Was Given
	{
		return NULL;									// If Not Return NULL
	}

	File=fopen(Filename,"r");							// Check To See If The File Exists

	if (File)											// Does The File Exist?
	{
		fclose(File);									// Close The Handle
		return auxDIBImageLoad(Filename);				// Load The Bitmap And Return A Pointer
	}

	return NULL;										// If Load Failed Return NULL
}

int LoadGLTextures()									// Load Bitmaps And Convert To Textures
{
	int Status=FALSE;									// Status Indicator

	AUX_RGBImageRec *TextureImage[1];					// Create Storage Space For The Texture

	memset(TextureImage,0,sizeof(void *)*1);           	// Set The Pointer To NULL

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/BlueGrain.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[0]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[0]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/wood5.bmp"))//data/cube.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[1]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[1]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}


	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/asphalt.bmp"))//data/bricksgreen.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[2]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[2]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}


/*	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/wood2_red.bmp"))//data/bricksred.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[3]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[3]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}
*/
	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/wood4.bmp"))//data/bricksred.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[4]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[4]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/PlayAgain.bmp"))//data/bricksred.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[5]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[5]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/Options.bmp"))//data/bricksred.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[6]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[6]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("texture/Exit.bmp"))//data/bricksred.bmp")) //"Data/Wall.bmp"))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[7]);					// Create Three Textures

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture[7]);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0])								// If Texture Exists
	{
		if (TextureImage[0]->data)						// If Texture Image Exists
		{
			free(TextureImage[0]->data);				// Free The Texture Image Memory
		}

		free(TextureImage[0]);							// Free The Image Structure
	}

	return Status;										// Return The Status
}

GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.5f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}


int frames = 0;

float Time1;
float Time2;
float DiffTime;
float FrameRate = 1;
float FrameRatep = 0;
float OverallSpeed = FrameRate;

void Benchmark()
{
//FPS /////////////////////////////
///////////////////////////////////
//
    int repeat = 50;
    Time1 = TimerGetTime()/1000;
    glLoadIdentity();
    glTranslatef(0,0,0);
    
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);	// Clear The Screen And The Depth Buffer         
    SwapBuffers(hDC);
    glEnable(GL_BLEND);
    for(float cnt = 0; cnt < repeat; cnt+=.1)
    {
      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);	// Clear The Screen And The Depth Buffer         
      glRotatef(cnt,1,0,0);
      glColor4f(1,1,1,cnt/10);
      glBegin(GL_TRIANGLE_STRIP);
      glTexCoord2f(1,1); glVertex3f(cnt+1,cnt+1,-20); 
      glTexCoord2f(1,0); glVertex3f(cnt+1,cnt,-20);
      glTexCoord2f(0,1); glVertex3f(cnt,cnt+1,-20);
      glTexCoord2f(0,0); glVertex3f(cnt,cnt,-20);
      glEnd();
      SwapBuffers(hDC);
    }  
    glDisable(GL_BLEND);
    
    Time2 = TimerGetTime()/1000;
    DiffTime = ABS(Time2-Time1);    
}

int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	if (!LoadGLTextures())								// Jump To Texture Loading Routine
	{
		return FALSE;									// If Texture Didn't Load Return FALSE
	}

	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping

	glClearColor(0.7f, 0.7f, 0.8f, 1.0f);				// Background Color
	glClearDepth(1.0f);		
    							// Depth Buffer Setup   
	glEnable(GL_DEPTH_TEST);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);					// Type Of Blending To Perform

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

	glFogi(GL_FOG_MODE, fogMode[1]);			// Fog Mode
	glFogfv(GL_FOG_COLOR, fogColor);					// Set Fog Color
	glFogf(GL_FOG_DENSITY, 0.05f);						// How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);					// Fog Hint Value
	glFogf(GL_FOG_START, 0.0f);							// Fog Start Depth
	glFogf(GL_FOG_END, 10.0f);							// Fog End Depth
	glEnable(GL_FOG);									// Enables GL_FOG

  float m_ambient[4] =  {.1,.1,.1,1};
  float m_diffuse[4] =  {.5,.5,.5,1};
  float m_specular[4] = {1,1,1,1};    
  float m_shininess = 10.f;    
	glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, m_ambient );
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, m_diffuse );	
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, m_specular );
	glMaterialf ( GL_FRONT_AND_BACK, GL_SHININESS, m_shininess );

  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE); 
	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);		// Setup The Ambient Light
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);		// Setup The Diffuse Light
	glLightfv(GL_LIGHT0, GL_SPECULAR,LightSpecular);		// Setup The Diffuse Light
  glLightfv(GL_LIGHT0, GL_POSITION,LightPosition);	// Position The Light
  //glLightf(GL_LIGHT1,GL_SPOT_CUTOFF,15.f);
  //glLightf(GL_LIGHT1,GL_SPOT_EXPONENT,.02f);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);

	quadratic=gluNewQuadric();							// Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
	gluQuadricNormals(quadratic, GLU_SMOOTH);			// Create Smooth Normals (NEW)
	gluQuadricTexture(quadratic, GL_TRUE);				// Create Texture Coords (NEW)

  TimerInit();
//Benchmark();       

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	return TRUE;										// Initialization Went OK
}

GLfloat x = 0;
GLfloat y = -1.75;
GLfloat	z = -15.0f;			// Depth Into The Screen

GLfloat xp = 1;
GLfloat yp = .9;
GLfloat zp = 1.3;

GLfloat mx = 0;
GLfloat my = -1.75;
GLfloat mz = -6+.25;
GLfloat mxp = 0;
GLfloat myp = 0;
GLfloat mzp = 0;


GLfloat cmx = 0;
GLfloat cmy = -1.75;
GLfloat cmz = -16-.25;
GLfloat cmxp = 0;
GLfloat cmyp = 0;
GLfloat cmzp = 0;

GLfloat r = 0;
GLfloat g = 0;
GLfloat b = 0;    

GLfloat	xrot;				// X Rotation
GLfloat	yrot;				// Y Rotation
GLfloat	zrot;				// Y Rotation
GLfloat xspeed = 0;				// X Rotation Speed
GLfloat yspeed = 0;				// Y Rotation Speed
GLfloat zspeed = 0;				// Y Rotation Speed

GLfloat RoomZrot = 0;

GLfloat Dist;
GLfloat cDist;
GLfloat Speed;

//xspeed = 0.5f;
//yspeed = 0.7f;
int cntclr = 0;

float alpha1=.25f;
float alpha2=.25f;
float alpha3=.25f;
float alpha1a=0;
float alpha2a=0;
float alpha3a=0;
float alpha1ap=0;
float alpha2ap=0;
float alpha3ap=0;

float menuZ = 0;

void Menu()
{    
    if (PlayAgain)
    {     
      alpha1+=.025f;
      alpha2-=.010f;
      alpha3-=.010f;
      if(alpha1 >   .75f) alpha1 = .75f;
      if(alpha2 < .5f) alpha2 = .5f;
      if(alpha3 < .5f) alpha3 = .5f;
      alpha1ap += .5f-alpha1a;
      alpha1a += alpha1ap/250;
      alpha2a=0; alpha2ap=0; alpha3a=0; alpha3ap=0;
    }
    else if (Options)
    {
      alpha1-=.010f;
      alpha2+=.025f;
      alpha3-=.010f;
      if(alpha1 < .5f) alpha1 = .5f;
      if(alpha2 >  .75f) alpha2 = .75f;
      if(alpha3 < .5f) alpha3 = .5f;
      alpha2ap += .5f-alpha2a;
      alpha2a += alpha2ap/250;
      alpha1a=0; alpha1ap=0; alpha3a=0; alpha3ap=0;
    }
    else if (Exit)
    {
      alpha1-=.010f;
      alpha2-=.010f;
      alpha3+=.025f;
      if(alpha1 < .5f) alpha1 = .5f;
      if(alpha2 < .5f) alpha2 = .5f;
      if(alpha3 > .75f) alpha3 = .75f;
      alpha3ap += .5f-alpha3a;
      alpha3a += alpha3ap/250;
      alpha1a=0; alpha1ap=0; alpha2a=0; alpha2ap=0;
    }
     
    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);

    int scalemenu = 2;
       
    menuZ-=.1;
    if (menuZ < -9) menuZ = -9;
    
    glLoadIdentity();
    glTranslatef(0,0,menuZ-1);    

//Backdrop
    glBindTexture(GL_TEXTURE_2D,texture[2]);
    glColor4f(0,0,1,1.f);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, (.8)*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu, (-.4)*scalemenu,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, (.8)*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu, (-.4)*scalemenu,0);
    glEnd();   


    glBindTexture(GL_TEXTURE_2D,texture[5]);
    glColor4f(1,1,1,(alpha1+alpha1a)/2);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu,(.4+.4)*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu,   .4*scalemenu,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu,(.4+.4)*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu,   .4*scalemenu,0);
    glEnd();  
    
    /*glColor4f(.5f,.5f,alpha1a,.5f);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, (.4+.4)*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu,    .4*scalemenu,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, (.4+.4)*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu,    .4*scalemenu,0);
    glEnd();  
    */
    glBindTexture(GL_TEXTURE_2D,texture[6]);
    glColor4f(1,1,1,(alpha2+alpha2a)/2);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, .4*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu,  0,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, .4*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu,  0,0);
    glEnd();   
    
    /*glColor4f(.5f,.5f,alpha2a,.5f);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, .4*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu,  0,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, .4*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu,  0,0);
    glEnd();*/

    glBindTexture(GL_TEXTURE_2D,texture[7]);
    glColor4f(1,1,1,(alpha3+alpha3a)/2);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, (.4-.4)*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu, (0-.4)*scalemenu,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, (.4-.4)*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu, (0-.4)*scalemenu,0);
    glEnd();   
    
    /*glColor4f(.5f,.5f,alpha3a,.5f);
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(1,1); glVertex3f( 1.0*scalemenu, (.4-.4)*scalemenu,0);
    glTexCoord2f(1,0); glVertex3f( 1.0*scalemenu, (0-.4)*scalemenu,0);
    glTexCoord2f(0,1); glVertex3f(-1.0*scalemenu, (.4-.4)*scalemenu,0);
    glTexCoord2f(0,0); glVertex3f(-1.0*scalemenu, (0-.4)*scalemenu,0);
    glEnd();*/
    //glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_LIGHTING);
    glEnable(GL_DEPTH_TEST);   
}
    

int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

  glLoadIdentity();									// Reset The View
	
  if(!MENU)
  {   
    r-=.01;
    g-=.01;
    if (r < 0) r = 0; 
    if (g < 0) g = 0; 

    Dist = Hypot(x-mx,z-mz);
    cDist = Hypot(x-cmx,z-cmz);

    if (z > 1) 
    {
      PlaySound("youlost.wav", NULL, SND_SYNC);
      x = 0;
      y = -1.75;
      z = -15.0f;			// Depth Into The Screen
      xp = 1;
      yp = .9;
      zp = 1.3;
      menuZ=0;
      MENU=true;
    }
    else if (z < -19) 
    {
      PlaySound("youwin.wav", NULL, SND_SYNC);
      x = 0;
      y = -1.75;
      z = -15.0f;			// Depth Into The Screen
      xp = 1;
      yp = .9;
      zp = 1.3;
      menuZ=0;
      MENU=true;
    } 
    if (zp > 0 && Dist < .75)
    {     
      PlaySound("tennis.wav", NULL, SND_ASYNC);
      g = 1; 
      
      zp = -zp;
         
      Speed = hypot(ABS(xp),ABS(zp));
      xp += ((x-mx) * (Speed*5)) / 3.14f; 
      
      xp += mxp;    
      zp += mzp*5;    
      if (xp > .25 || xp < -.25) xp *= .99; 
      if (zp > .25 || zp < -.25) zp *= .99; 
      z += zp /10;
    }

    else if (zp < 0 && cDist < .75)// && z >= cmz && x > cmx -.5-.15 && x < cmx +.5+.15) 
    {
      PlaySound("tennis.wav", NULL, SND_ASYNC);
      r = 1; 
      
      zp = -zp;
      z += zp /10;
    }

    if (x > 2.5) x = 2.5, xp = -xp,PlaySound("tennis1.wav", NULL, SND_ASYNC);    
    else if (x < -2.5) x = -2.5, xp = -xp,PlaySound("tennis1.wav", NULL, SND_ASYNC); 

    x += xp/25;
    z += zp/25;
     
    mxp *= .95;
    mzp *= .93;
   
    mx += mxp; 
    mz += mzp; 

    if (mx >  2.25) mx =  2.25,mxp = 0;
    if (mx < -2.25) mx = -2.25,mxp = 0;  
    if (mz < -7) mz = -7, mzp = 0;
    if (mz > -6+.25) mz = -6+.25, mzp = 0;

    if (cmx >  2.25) cmx =  2.25,cmxp = 0;
    if (cmx < -2.25) cmx = -2.25,cmxp = 0;  

    cmxp += (x-cmx)/5;
    cmxp *= .95;
    cmx += cmxp/25; 
  }
  GLfloat LightDiffuse1[]=		{ 0.5f+r, 0.5f+g, 0.5f+b, 1.0f };

	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse1);		// Setup The Diffuse Light
//    cmx = x; // AI Is the BOMB! // Comment to disable

  // 3D ROOM //////////////////
  glLoadIdentity();
  glTranslatef(0,-1.75,-30);
  glColor4f(.25,.25,.25,1.f); 
  glBindTexture(GL_TEXTURE_2D, texture[4]);

	RoomZrot += 2;
	glRotatef(RoomZrot,0.0f,0.0f,1.0f);
  glColor3f(.75,.75,.75); 
    
  gluCylinder(quadratic,2.75f,2.75f,30,32,1);	// A Cylinder With A Bottom Radius Of .5 And A Height Of 2

  glEnable(GL_BLEND);
  glDisable(GL_DEPTH_TEST);

  glLoadIdentity();
  glColor4f(.75,.75,.75,1.f); 
//  3D ROOM 
  glBindTexture(GL_TEXTURE_2D, texture[1]);
  glBegin(GL_QUADS);
  // Bottom Face
	glNormal3f( 0,1, 0);
	glTexCoord2f(1, 1); glVertex3f(-2.75, -2.0, -30);
	glTexCoord2f(0, 1); glVertex3f( 2.75, -2.0, -30);
	glTexCoord2f(0, 0); glVertex3f( 2.75, -2.0,  0);
	glTexCoord2f(1, 0); glVertex3f(-2.75, -2.0,  0);
	glEnd();

  glDisable(GL_LIGHTING);
  glLoadIdentity(); 
  glTranslatef(x,-2.25,z);
  
	glRotatef(180,0.0f,0.0f,1.0f);
  glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(zrot,0.0f,0.0f,1.0f);

    // reflected PUCK
	glColor4f(.25,.25,.25,.25);
  glBindTexture(GL_TEXTURE_2D, texture[0]);
  gluSphere(quadratic,.25,16,16);				// Draw A Sphere With A Radius Of 1 And 16 Longitude And 16 Latitude Segments
  glEnable(GL_LIGHTING);

  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);

  glLoadIdentity(); 
  glTranslatef(x,y,z);
 
	glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(zrot,0.0f,0.0f,1.0f);

  // PUCK
	glColor3f(1,1,1);
  gluSphere(quadratic,.25,16,16);				// Draw A Sphere With A Radius Of 1 And 16 Longitude And 16 Latitude Segments
   
  // YOUR PADDLE (CYLINDER) //////////
	glLoadIdentity();									// Reset The View   
  glColor3f(.5,1,.5); 
	glBindTexture(GL_TEXTURE_2D, texture[2]);
  glTranslatef(mx,my,mz);
  glRotatef(180,1.0f,1.0f,0.0f);
  gluSphere(quadratic,.5,2,32);				// Draw A Sphere With A Radius Of 1 And 16 Longitude And 16 Latitude Segments
	glLoadIdentity();									// Reset The View
  glTranslatef(mx,my,mz);
  glRotatef(90,1.0f,0.0f,0.0f);
  gluCylinder(quadratic,0.5f,0.5f,0.3f,32,1);	// A Cylinder With A Bottom Radius Of .5 And A Height Of 2

  // OPPONENT'S PADDLE (CYLINDER)	
  glLoadIdentity();									// Reset The View
	glColor3f(1,.5,.5); 
  glTranslatef(cmx,cmy,cmz);
  glRotatef(180,1.0f,1.0f,0.0f);
  gluSphere(quadratic,.5,2,16);				// Draw A Sphere With A Radius Of 1 And 16 Longitude And 16 Latitude Segments
	glLoadIdentity();									// Reset The View
  glTranslatef(cmx,cmy,cmz);
  glRotatef(90,1.0f,0.0f,0.0f);
  gluCylinder(quadratic,0.5f,0.5f,0.3f,16,1);	// A Cone With A Bottom Radius Of .5 And A Height Of 2

  xspeed += zp/5;
  if (xspeed > ABS(zp*5)) xspeed = ABS(zp*5); 
  if (xspeed < -ABS(zp*5)) xspeed = -ABS(zp*5);
  zspeed += xp/5;
  if (zspeed > ABS(xp*5)) zspeed = ABS(xp*5); 
  if (zspeed < -ABS(xp*5)) zspeed = -ABS(xp*5);
 	xspeed *= .99;
	zspeed *= .99;
  xrot+=xspeed;
	zrot-=zspeed;

  Time2 = TimerGetTime()/1000;
  DiffTime = ABS(Time2-Time1);      
  while (DiffTime < .015f) // .015 = 66 frames per second
  {
    Sleep(1);
    Time2 = TimerGetTime()/1000;
    DiffTime = ABS(Time2-Time1);      
  }
  Time1 = TimerGetTime()/1000;     
    
  return TRUE;										// Keep Going
}

GLvoid KillGLWindow(GLvoid)								// Properly Kill The Window
{
	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (fullscreen)										// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	HINSTANCE	hInstance;				// Holds The Instance Of The Application
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;						// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
	}

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,			// Extended Style For The Window
								"OpenGL",			// Class Name
								title,				// Window Title
								dwStyle,			// Window Style
								0, 0,				// Window Position
								width, height,		// Selected Width And Height
								NULL,				// No Parent Window
								NULL,				// No Menu
								hInstance,			// Instance
								NULL)))				// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

long CNTR = 0;


int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

    CNTR++;
	// Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;							// Windowed Mode
	}

	// Create Our OpenGL Window
	if (!CreateGLWindow("Steve's Pong Mania",800,600,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	while(!done && !DONE)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
		   // Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !DrawGLScene()))	// Active?  Was There A Quit Received?
			{			
                  done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{
            if (MENU) Menu();            
  			SwapBuffers(hDC);					// Swap Buffers (Double Buffering)
					
				if (keys[VK_ESCAPE] && !esc)
	            {
	              esc=true;
                  menuZ=0;
                  MENU=!MENU;                 
	            }
                if (!keys[VK_ESCAPE])
                {
                  esc=false;    
                }

              if (MENU) 
              {

				if (keys[VK_UP] && !up)
	            {
	              PlaySound("type.wav", NULL, SND_ASYNC);
                  up=true;
                  item--;
                  if (item < 1) item = 3;                 
	            }
                if (!keys[VK_UP])
                {
                  up=false;    
                }
	            if (keys[VK_DOWN] && !dn)
	            {
	              PlaySound("type.wav", NULL, SND_ASYNC);
                  dn=true;
                  item++;
                  if (item > 3) item = 1;                 
	            }
                if (!keys[VK_DOWN])
                {
                  dn=false;    
                }
                if (keys[VK_RETURN])
                {
                  rn=true;
                  Selection = true;
                  if (PlayAgain)
                    MENU=!MENU;
                  else if (Exit)
                    done=true;
                }
                if (!keys[VK_RETURN])
                {
                  rn=false;    
                }

                  
                  switch(item)
                  {
                    case 1: PlayAgain=true,  Options=false, Exit=false; break;
                    case 2: PlayAgain=false, Options=true,  Exit=false; break;
                    case 3: PlayAgain=false, Options=false, Exit=true; break;
                  }

  			//SwapBuffers(hDC);					// Swap Buffers (Double Buffering)

              }
              else
              {

                if (keys[VK_UP])
				{
					mzp -= .0055;
                    //xspeed-=0.01f;
				}
				if (keys[VK_DOWN])
				{
					mzp += .0055;
				    //xspeed+=0.01f;
				}
				if (keys[VK_RIGHT])
				{
					mxp += .0055;
					//yspeed+=0.01f;
				}
				if (keys[VK_LEFT])
				{
					mxp -= .0055;		
                    //yspeed-=0.01f;
				}
                
              }
			}
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}
